/*
 * JavaBotHard.java
 */

package JavaBotHard;

import java.io.*;
import EDU.gatech.cc.is.abstractrobot.*;
import EDU.gatech.cc.is.clay.*;
import EDU.gatech.cc.is.util.*;

/**
 * Runs a control system on a Nomad 150 (and soon other hardware).
 * It intializes the robot,
 * then calls the control system periodically to
 * drive the robot.
 * <P>
 * To run this program, first ensure you are in the correct directory (where
 * the JavaBotHard.class file is), then type "java JavaBotHard".
 * <P>
 * <A HREF="../COPYRIGHT.html">Copyright</A>
 * (c)1997, 1998 Tucker Balch
 *
 * @author Tucker Balch
 * @version $Revision: 1.10 $
 * @see EDU.gatech.cc.is.abstractrobot.CommN150Hard
 */

public class JavaBotHard
	{

        /**
	 * Main for JavaBotHard.
         */
	public static void main(String[] args)
		{
		MultiForageN150HardPassiveGrip abstract_robot = null;
		ControlSystemMFN150 cs = new ControlSystemMFN150();
		boolean keep_going = true;
		int runtime = 0;
		double	startx = 0;
		double	starty = 0;
		String  server;
		int id = 0;

		/*--- check the arguments first ---*/
		if (args.length >= 1)
			{
			if (args[0].equalsIgnoreCase("-version"))
                               {
                               System.out.println(
                                        JavaBotsVersion.longReport());
                                System.exit(0);
                               }
			}
		System.out.println(JavaBotsVersion.shortReport());
		if (args.length < 6)
			{
			System.out.println("usage: java JavaBotHard [-version] controlsystem commserver id secondstorun x y");
			System.exit(1);
			}

		/*--- get server ---*/
		server = args[1];
	
		/*--- check id ---*/
		try
			{
			id = Integer.valueOf(args[2]).intValue();
			}
		catch (NumberFormatException e)
			{
			System.out.println(args[2] + " is not a valid id.");
			System.exit(1);
			}
		if (id <= 0)
			{
			System.out.println(args[2] + " is not a valid id.");
			System.exit(1);
			}

		/*--- check run time ---*/
		try
			{
			runtime = 1000*Integer.valueOf(args[3]).intValue();
			}
		catch (NumberFormatException e)
			{
			System.out.println(args[3] + " is not a valid number of seconds.");
			System.exit(1);
			}
		if (runtime <= 0)
			{
			System.out.println(args[3] + " is not a valid number of seconds.");
			System.exit(1);
			}

		/*--- get the starting position ---*/
		try
			{
			startx = Double.valueOf(args[4]).doubleValue();
			starty = Double.valueOf(args[5]).doubleValue();
			}
		catch (NumberFormatException e)
			{
			System.out.println("("+args[4]+","+args[5]+")"
				+" is not a valid starting position.");
			System.exit(1);
			}

		/*--- load the control system ---*/
		try
			{
			Class csclass = Class.forName(args[0]);
			cs = (ControlSystemMFN150)csclass.newInstance();
			}
		catch (ClassNotFoundException e )
			{
			System.out.println(args[0]+
				" was not found.");
			System.exit(1);
			}
		catch (IllegalAccessException e)
			{
			System.out.println(args[0]+
				" is not a ControlSystemMFN150 object.");
			System.exit(1);
			}
		catch (InstantiationException e)
			{
			System.out.println(args[0]+
				" could not be instantiated.");
			System.exit(1);
			}

		try
			{
			/*--- inform the user ---*/
			System.out.println("JavaBotHard: initializing the robot.");
			/*--- open the connection to the robot hardware ---*/
			try
				{
				abstract_robot = new 
					MultiForageN150HardPassiveGrip(
						1,38400);
					//MultiForageN150HardPassiveGrip(
						//1,38400,server,id);
				//abstract_robot = new CommN150Hard(1,38400);
				}
			catch(Exception e)
				{
				System.out.println(e);
				}
				
			abstract_robot.setBaseSpeed(abstract_robot.MAX_TRANSLATION);
			abstract_robot.resetPosition(new Vec2(startx, starty));

			/*--- initialize the control system ---*/
			cs.init(abstract_robot,System.currentTimeMillis());
			cs.configure();
			cs.trialInit();
			}
		catch(Exception e)
			{
			System.out.println(e);
			System.exit(1);
			}

		/*--- wait to begin ---*/
		//System.out.println("JavaBotHard: ready to go, hit return to start.");
		//int inchar = 0;
		//while(inchar != 10)
			//{
			//try{inchar = System.in.read();}
			//catch(Exception e){}
			//}

		/*--- get the time we started ---*/
		long start_time = abstract_robot.getTime();
		long curr_time = start_time;
		double cycles = 0;

		/*--- loop until done ---*/
		int ret_val = ControlSystemMFN150.CSSTAT_OK;
		System.out.println("******STARTING******");
		while (((curr_time - start_time)<runtime)&&
			(ret_val == ControlSystemMFN150.CSSTAT_OK))
			{
			// think about what to do
			ret_val = cs.takeStep();

			// let the hardware run
			abstract_robot.takeStep();

			// let other threads run
			// NOPE! it runs synchronously now
			//try{Thread.sleep(50);}
			//catch(Exception e){}

			// garbage collect (for uniform cycle times)
			System.gc();

			// keep track of time
			cycles++;
			curr_time = abstract_robot.getTime();
			}
		curr_time = abstract_robot.getTime();
		System.out.println("******STOPPING******");

		/*--- shutdown ---*/
		cs.trialEnd();
		cs.quit();
		abstract_robot.quit(); // only do this when done!
		System.out.println("JavaBotHard: "+
			(cycles*1000/(double)curr_time)+ 
			" control cycles per second.");

		}
/*
	catch(Exception e)
		{
		System.out.println(e);
		System.exit(1);
		}
*/
	}
